
using System;
using System.Collections.Generic;

namespace Calculator
{
	/// <summary>
	/// A specific visitor meant to evaluate expressions in rpn notation
	/// </summary>

	public class Evaluator:IVisitor
	{

		private Stack<INode> innerStack;
		
		private double result;
		
		/// <summary>
		/// Return the result of an evaluation on the stack
		/// </summary>
		/// <returns>
		/// A <see cref="System.Double"/>
		/// </returns>
		public double Result
		{
			get	{ return result;	}
		}
		/// <summary>
		/// Costruct an evaluator given a stack of operands and operators
		/// </summary>
		/// <param name="st">
		/// A <see cref="Stack<INode>"/>
		/// </param>
		public Evaluator (Stack<INode> st)
		{
			innerStack = st;
		}
		
		/// <summary>
		/// Visit the operand on the stack, update result
		/// </summary>
		/// <param name="op">
		/// A <see cref="Operand"/>
		/// </param>
		public void Visit(Operand op)
		{
			result = op.InternalValue;
		}
		//Helper method introduced to factor out code
		private double GetNextOperand()
		{
			INode n;
			try
			{		
				n = innerStack.Pop();
				n.Accept(this);
				return result;
			}
			catch(Exception ese)
			{
				Console.Error.Write(ese.Message);
				throw new IllegalNumberOfOperandsException ();
			}
		}
		/// <summary>
		/// Visit operator "+", update result
		/// </summary>
		/// <param name="opt">
		/// A <see cref="PlusOperator"/>
		/// </param>
		public void Visit(PlusOperator opt)
		{
			double d1;
			double d2;
			d1 = GetNextOperand();
			d2 = GetNextOperand();
			result = d1 + d2;
		}
		/// <summary>
		/// Visit operator "-", update result
		/// </summary>
		/// <param name="opt">
		/// A <see cref="MinusOperator"/>
		/// </param>
		public void Visit(MinusOperator opt)
		{
			double d1;
			double d2;
			d1 = GetNextOperand();
			d2 = GetNextOperand();
			result = d2 - d1;	
		}
		/// <summary>
		/// Visit operator "*", update result
		/// </summary>
		/// <param name="opt">
		/// A <see cref="TimesOperator"/>
		/// </param>
		public void Visit(TimesOperator opt)
		{
			double d1;
			double d2;
			d1 = GetNextOperand();
			d2 = GetNextOperand();
			result = d2 * d1;	
		}
		/// <summary>
		/// Visit operator "/", update result
		/// </summary>
		/// <param name="opt">
		/// A <see cref="DivideByOperator"/>
		/// </param>
		public void Visit(DivideByOperator opt)
		{
			double d1;
			double d2;
			d1 = GetNextOperand();
			d2 = GetNextOperand();
			result = d2 / d1;		
		}
	}
}
